@@ -19,7 +19,7 @@
 #
 #
 from collections import OrderedDict
-from typing import Any, Optional
+from typing import Any
 
 from twisted.internet.testing import MemoryReactor
 
@@ -232,7 +232,7 @@ class FederationKnockingTestCase(
         # Have this homeserver skip event auth checks. This is necessary due to
         # event auth checks ensuring that events were signed by the sender's homeserver.
         async def _check_event_auth(
-            origin: Optional[str], event: EventBase, context: EventContext
+            origin: str | None, event: EventBase, context: EventContext
         ) -> None:
             pass
 
