@@ -20,7 +20,6 @@
 #
 #
 import os
-from typing import Dict
 
 from parameterized import parameterized
 
@@ -51,7 +50,7 @@ class _AdminMediaTests(unittest.HomeserverTestCase):
         media.register_servlets,
     ]
 
-    def create_resource_dict(self) -> Dict[str, Resource]:
+    def create_resource_dict(self) -> dict[str, Resource]:
         resources = super().create_resource_dict()
         resources["/_matrix/media"] = self.hs.get_media_repository_resource()
         return resources
