@@ -18,7 +18,6 @@
 # [This file includes modifications made by New Vector Limited]
 #
 #
-from typing import Optional
 
 from twisted.internet.testing import MemoryReactor
 
@@ -44,8 +43,8 @@ class UsernameAvailableTestCase(unittest.HomeserverTestCase):
 
         async def check_username(
             localpart: str,
-            guest_access_token: Optional[str] = None,
-            assigned_user_id: Optional[str] = None,
+            guest_access_token: str | None = None,
+            assigned_user_id: str | None = None,
             inhibit_user_in_use_error: bool = False,
         ) -> None:
             if localpart == "allowed":
