@@ -46,7 +46,7 @@ from synapse.util.clock import Clock
 from synapse.util.stringutils import random_string
 
 from tests import unittest
-from tests.server import TimedOutException
+from tests.server import FakeChannel, TimedOutException
 from tests.test_utils.event_injection import create_event
 
 logger = logging.getLogger(__name__)
@@ -80,12 +80,10 @@ class SlidingSyncBase(unittest.HomeserverTestCase):
         config["experimental_features"] = {"msc3575_enabled": True}
         return config
 
-    def do_sync(
+    def make_sync_request(
         self, sync_body: JsonDict, *, since: str | None = None, tok: str
-    ) -> tuple[JsonDict, str]:
-        """Do a sliding sync request with given body.
-
-        Asserts the request was successful.
+    ) -> FakeChannel:
+        """Make a sliding sync request with given body.
 
         Attributes:
             sync_body: The full request body to use
@@ -106,6 +104,24 @@ class SlidingSyncBase(unittest.HomeserverTestCase):
             content=sync_body,
             access_token=tok,
         )
+        return channel
+
+    def do_sync(
+        self, sync_body: JsonDict, *, since: str | None = None, tok: str
+    ) -> tuple[JsonDict, str]:
+        """Do a sliding sync request with given body.
+
+        Asserts the request was successful.
+
+        Attributes:
+            sync_body: The full request body to use
+            since: Optional since token
+            tok: Access token to use
+
+        Returns:
+            A tuple of the response body and the `pos` field.
+        """
+        channel = self.make_sync_request(sync_body, since=since, tok=tok)
         self.assertEqual(channel.code, 200, channel.json_body)
 
         return channel.json_body, channel.json_body["pos"]
