@@ -23,7 +23,7 @@ import os
 import re
 from email.parser import Parser
 from http import HTTPStatus
-from typing import Any, Optional, Union
+from typing import Any
 from unittest.mock import Mock
 
 from twisted.internet.interfaces import IReactorTCP
@@ -363,7 +363,7 @@ class PasswordResetTestCase(unittest.HomeserverTestCase):
         email: str,
         client_secret: str,
         ip: str = "127.0.0.1",
-        next_link: Optional[str] = None,
+        next_link: str | None = None,
     ) -> str:
         body = {"client_secret": client_secret, "email": email, "send_attempt": 1}
         if next_link is not None:
@@ -384,7 +384,7 @@ class PasswordResetTestCase(unittest.HomeserverTestCase):
 
         return channel.json_body["sid"]
 
-    def _validate_token(self, link: str, next_link: Optional[str] = None) -> None:
+    def _validate_token(self, link: str, next_link: str | None = None) -> None:
         # Remove the host
         path = link.replace("https://example.com", "")
 
@@ -1152,9 +1152,9 @@ class ThreepidEmailRestTestCase(unittest.HomeserverTestCase):
         self,
         email: str,
         client_secret: str,
-        next_link: Optional[str] = None,
+        next_link: str | None = None,
         expect_code: int = HTTPStatus.OK,
-    ) -> Optional[str]:
+    ) -> str | None:
         """Request a validation token to add an email address to a user's account
 
         Args:
@@ -1394,10 +1394,10 @@ class AccountStatusTestCase(unittest.HomeserverTestCase):
         async def post_json(
             destination: str,
             path: str,
-            data: Optional[JsonDict] = None,
+            data: JsonDict | None = None,
             *a: Any,
             **kwa: Any,
-        ) -> Union[JsonDict, list]:
+        ) -> JsonDict | list:
             if destination == "remote":
                 return {
                     "account_statuses": {
@@ -1503,11 +1503,11 @@ class AccountStatusTestCase(unittest.HomeserverTestCase):
 
     def _test_status(
         self,
-        users: Optional[list[str]],
+        users: list[str] | None,
         expected_status_code: int = HTTPStatus.OK,
-        expected_statuses: Optional[dict[str, dict[str, bool]]] = None,
-        expected_failures: Optional[list[str]] = None,
-        expected_errcode: Optional[str] = None,
+        expected_statuses: dict[str, dict[str, bool]] | None = None,
+        expected_failures: list[str] | None = None,
+        expected_errcode: str | None = None,
     ) -> None:
         """Send a request to the account status endpoint and check that the response
         matches with what's expected.
