@@ -19,7 +19,6 @@
 #
 #
 
-from typing import Optional
 
 from twisted.internet.testing import MemoryReactor
 
@@ -345,9 +344,7 @@ class EventPushActionsStoreTestCase(HomeserverTestCase):
                 aggregate_counts[room_id], notif_count + thread_notif_count
             )
 
-        def _create_event(
-            highlight: bool = False, thread_id: Optional[str] = None
-        ) -> str:
+        def _create_event(highlight: bool = False, thread_id: str | None = None) -> str:
             content: JsonDict = {
                 "msgtype": "m.text",
                 "body": user_id if highlight else "msg",
@@ -527,9 +524,7 @@ class EventPushActionsStoreTestCase(HomeserverTestCase):
                 aggregate_counts[room_id], notif_count + thread_notif_count
             )
 
-        def _create_event(
-            highlight: bool = False, thread_id: Optional[str] = None
-        ) -> str:
+        def _create_event(highlight: bool = False, thread_id: str | None = None) -> str:
             content: JsonDict = {
                 "msgtype": "m.text",
                 "body": user_id if highlight else "msg",
@@ -553,7 +548,7 @@ class EventPushActionsStoreTestCase(HomeserverTestCase):
         def _rotate() -> None:
             self.get_success(self.store._rotate_notifs())
 
-        def _mark_read(event_id: str, thread_id: Optional[str] = None) -> None:
+        def _mark_read(event_id: str, thread_id: str | None = None) -> None:
             self.get_success(
                 self.store.insert_receipt(
                     room_id,
